module Model exposing (..)

import Set exposing (Set)
import Dict exposing (Dict)
import Json.Decode as Decode exposing (..)
import Json.Decode.Extra exposing ((|:), withDefault)
import Json.Encode as Encode exposing (..)


type AuthRequest
    = AuthRequest
        { username : String
        , password : String
        }


authRequestEncoder : AuthRequest -> Encode.Value
authRequestEncoder (AuthRequest model) =
    [ (\username -> ( "username", Encode.string username )) model.username
    , (\password -> ( "password", Encode.string password )) model.password
    ]
        |> Encode.object


authRequestDecoder : Decoder AuthRequest
authRequestDecoder =
    (Decode.succeed
        (\username password ->
            AuthRequest
                { username = username
                , password = password
                }
        )
    )
        |: (field "username" Decode.string)
        |: (field "password" Decode.string)


type RefreshRequest
    = RefreshRequest
        { refreshToken : String
        }


refreshRequestEncoder : RefreshRequest -> Encode.Value
refreshRequestEncoder (RefreshRequest model) =
    [ (\refreshToken -> ( "refreshToken", Encode.string refreshToken )) model.refreshToken
    ]
        |> Encode.object


refreshRequestDecoder : Decoder RefreshRequest
refreshRequestDecoder =
    (Decode.succeed
        (\refreshToken ->
            RefreshRequest
                { refreshToken = refreshToken
                }
        )
    )
        |: (field "refreshToken" Decode.string)


type AuthResponse
    = AuthResponse
        { token : String
        , refreshToken : String
        }


authResponseEncoder : AuthResponse -> Encode.Value
authResponseEncoder (AuthResponse model) =
    [ (\token -> ( "token", Encode.string token )) model.token
    , (\refreshToken -> ( "refreshToken", Encode.string refreshToken )) model.refreshToken
    ]
        |> Encode.object


authResponseDecoder : Decoder AuthResponse
authResponseDecoder =
    (Decode.succeed
        (\token refreshToken ->
            AuthResponse
                { token = token
                , refreshToken = refreshToken
                }
        )
    )
        |: (field "token" Decode.string)
        |: (field "refreshToken" Decode.string)
